import 'package:equatable/equatable.dart';

/// Version of the touch events format expected by the native API
const int version = 1;

/// Types of the different gesture supported by the Zoning feature
enum CSGesture {
  /// tap - 6
  tap,

  /// longsPres - 8
  longPress,

  /// drag - 9
  drag,

  /// flick - 10
  flick,
}

/// Extension to map the int representation of a [CSGesture]
extension CSGestureInt on CSGesture {
  /// Return the int representation for the [CSGesture]
  int toInt() {
    switch (this) {
      case CSGesture.tap:
        return 6;
      case CSGesture.longPress:
        return 8;
      case CSGesture.drag:
        return 9;
      case CSGesture.flick:
        return 10;
    }
  }
}

/// Direction of a CSGesture
/// this concerns the Swipe (Drag/Flick) : UP, DOWN, LEFT, RIGHT
enum CSGestureDirection {
  /// up - 1
  up,

  /// down - 2
  down,

  /// left - 3
  left,

  /// right - 4
  right,
}

/// Extension to map the int representation of a [CSGestureDirection]
extension CSGestureDirectionInt on CSGestureDirection {
  /// Return the int representation for the [CSGestureDirection]
  int toInt() {
    switch (this) {
      case CSGestureDirection.up:
        return 1;
      case CSGestureDirection.down:
        return 2;
      case CSGestureDirection.left:
        return 3;
      case CSGestureDirection.right:
        return 4;
    }
  }
}

/// Struct to hold gesture related data
class CSGestureData extends Equatable {
  /// int - dragged/scrolled distance
  final int distance;

  /// CSGestureDirection (mapped to int) - finger direction - [1,2,3,4] -> [up, down, left, right] (see [CSGesture])
  final CSGestureDirection direction;

  /// int - velocity while scrolling
  final int velocity;

  /// Constructor for the CSGestureData data struct
  const CSGestureData({
    required this.distance,
    required this.direction,
    required this.velocity,
  });

  @override
  List<Object> get props => [distance, direction, velocity];
}

/// Struct to hold data for the touch events
class TouchEventData {
  /// String - target path inside the flutter widgets hierarchy
  final String path;

  /// CSGestureData - optional gesture related data
  final CSGestureData? gestureData;

  /// Constructor for the TouchEventData data struct
  TouchEventData({
    required this.path,
    this.gestureData,
  });

  /// Cast TouchEventData to Map<String, dynamic> for platform channel serialization
  Map<String, dynamic> asMap() {
    return {
      'path': path,
      'distance': gestureData?.distance,
      'direction': gestureData?.direction.toInt(),
      'velocity': gestureData?.velocity,
    };
  }
}
